import { ISortedMap, Key } from './sortedmap';
export interface ILruCache<K, V> extends ISortedMap<K, LruWrapper<V>> {
    nextOrder: number;
}
interface LruWrapper<V> {
    value: V;
    order: number;
}
export declare class LruCacheAdapter<K extends Key, V> {
    private suggestedSize;
    private sortedMapAdapter;
    /**
     * Create a new LRU Cache adapter.
     * @param suggestedSize The max suggested # of entries for the LRU Cache to store. Up to 10% more than the provided
     *                      number may be stored.
     */
    constructor(suggestedSize: number);
    create(): ILruCache<K, V>;
    set(lru: ILruCache<K, V>, key: K, value: V): void;
    get(lru: ILruCache<K, V>, key: K): V | undefined;
    getIterable(lru: ILruCache<K, V>): Iterable<{
        key: K;
        value: V;
    }>;
    getValuesIterable(lru: ILruCache<K, V>): Iterable<V>;
    update(lru: ILruCache<K, V>, key: K, updater: (item: V) => V | void): V | undefined;
    getSize(lru: ILruCache<K, V>): number;
    remove(lru: ILruCache<K, V>, key: K): void;
    private getNextOrder;
}
export {};
